#include <nsIDOMNodeList.h>
#include <nsIArray.h>
#include <nsArray.h>
#include <nsIDOMNode.h>
#include <nsString.h>

nsresult
FilterNodeListByType(nsIDOMNodeList* nodelist, nsIArray** oarray,
                     PRUint16 keepType)
{
  nsIMutableArray *array;
  NS_NewArray(&array);

  PRUint32 len;
  nodelist->GetLength(&len);
  for (PRUint32 i = 0; i < len; i++)
  {
    nsCOMPtr<nsIDOMNode> dn;
    nodelist->Item(i, getter_AddRefs(dn));
    PRUint16 nodeType;
    if (dn->GetNodeType(&nodeType) != NS_OK)
      continue;
    if (nodeType != keepType)
      continue;
    array->AppendElement(dn, PR_FALSE);
  }

  // All ownership is handled by the caller...
  *oarray = array;

  return NS_OK;
}

nsresult
FilterNodeListByTypeNS(nsIDOMNodeList* nodelist, nsIArray** oarray,
                       PRUint16 keepType,
                       const nsAString& ns)
{
  nsIMutableArray *array;
  NS_NewArray(&array);

  PRUint32 len;
  nodelist->GetLength(&len);
  for (PRUint32 i = 0; i < len; i++)
  {
    nsCOMPtr<nsIDOMNode> dn;
    nodelist->Item(i, getter_AddRefs(dn));
    PRUint16 nodeType;
    if (dn->GetNodeType(&nodeType) != NS_OK)
      continue;
    if (nodeType != keepType)
      continue;

    nsAutoString xname;
    dn->GetNamespaceURI(xname);
    if (!xname.Equals(ns))
      continue;

    array->AppendElement(dn, PR_FALSE);
  }

  // All ownership is handled by the caller...
  *oarray = array;

  return NS_OK;
}

nsresult
FilterNodeListByNameAndTypeNS(
                              nsIDOMNodeList* nodelist,
                              nsIArray** oarray,
                              PRUint16 keepType,
                              const nsAString& ns,
                              const nsAString& nname
                             )
{
  nsIMutableArray *array;
  NS_NewArray(&array);

  PRUint32 len;
  nodelist->GetLength(&len);
  for (PRUint32 i = 0; i < len; i++)
  {
    nsCOMPtr<nsIDOMNode> dn;
    nodelist->Item(i, getter_AddRefs(dn));
    PRUint16 nodeType;
    if (dn->GetNodeType(&nodeType) != NS_OK)
      continue;
    if (nodeType != keepType)
      continue;

    nsAutoString xname, nname2;
    nname2 = nname;
    dn->GetLocalName(xname);
    if (!xname.Equals(nname2))
      continue;
    dn->GetNamespaceURI(xname);
    if (!xname.Equals(ns))
      continue;

    array->AppendElement(dn, PR_FALSE);
  }

  // All ownership is handled by the caller...
  *oarray = array;

  return NS_OK;
}

nsresult
FilterNodeListByNameAndTypeNS2(
                              nsIDOMNodeList* nodelist,
                              nsIArray** oarray,
                              PRUint16 keepType,
                              const nsAString& ns1,
                              const nsAString& ns2,
                              const nsAString& nname
                             )
{
  nsIMutableArray *array;
  NS_NewArray(&array);

  PRUint32 len;
  nodelist->GetLength(&len);
  for (PRUint32 i = 0; i < len; i++)
  {
    nsCOMPtr<nsIDOMNode> dn;
    nodelist->Item(i, getter_AddRefs(dn));
    PRUint16 nodeType;
    if (dn->GetNodeType(&nodeType) != NS_OK)
      continue;
    if (nodeType != keepType)
      continue;

    nsAutoString xname, nname2;
    nname2 = nname;
    dn->GetLocalName(xname);
    if (!xname.Equals(nname2))
      continue;
    dn->GetNamespaceURI(xname);
    if (!xname.Equals(ns1) &&
        !xname.Equals(ns2))
      continue;

    array->AppendElement(dn, PR_FALSE);
  }

  // All ownership is handled by the caller...
  *oarray = array;

  return NS_OK;
}
