/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef WIN32
#include <signal.h>
#include <fenv.h>

#define SKIP_TRAP_INCREMENT fedisableexcept(FE_DIVBYZERO)
#define SKIP_TRAP_DECREMENT feclearexcept(FE_DIVBYZERO); feenableexcept(FE_DIVBYZERO)

#else

#define SKIP_TRAP_INCREMENT
#define SKIP_TRAP_DECREMENT
#endif

static double
arbitrary_log(double value, double base)
{
  if (base == 10.0)
    return log10(value);
  else if (base == M_E)
    return log(value);
  return log(value)/log(base);
}

static double
get_remainder(double div, double quo)
{
  return div - floor(div/quo)*quo;
}

static double
logical_xor(double a, double b)
{
  if ((a && b) || !(a || b))
    return 0.0;
  return 1.0;
}

static double
logical_implies(double a, double b)
{
  if (!a || (a && b))
    return 1.0;
  return 0.0;
}

static double
factorial(double f)
{
  double ret = 0;
  while (f > 0)
    ret *= (--f);
  return ret;
}

static PRInt32
imax(PRInt32 a1,
     PRInt32 a2)
{
  return (a1 > a2) ? a1 : a2;
}

static PRInt32
imin(PRInt32 a1,
     PRInt32 a2)
{
  return (a1 < a2) ? a1 : a2;
}

static double
max(double a1,
    double a2)
{
  if (!isfinite(a1))
    return a2;
  if (!isfinite(a2))
    return a1;
  return (a1 > a2) ? a1 : a2;
}

static double
min(double a1,
    double a2)
{
  if (!isfinite(a1))
    return a2;
  if (!isfinite(a2))
    return a1;
  return (a1 < a2) ? a1 : a2;
}

static double
abs_d(double a1)
{
  if (a1 < 0)
    return -a1;
  return a1;
}

static int
compare_with_tolerance(double a1, double a2)
{
  double nf;
  /* This covers not a number etc... too. */
  if (a1 == a2)
  {
    return 1;
  }
  if (a1 == 0.0 || a2 == 0.0)
    return 0;
  if (a1 < 0.0)
  {
    if (a2 > 0.0)
      return 0;
    a1 = -a1;
    a2 = -a2;
  }
  else if (a2 < 0.0)
    return 0;
  nf = a1;
  if (nf > a2)
    nf = a2;
  nf = abs_d((a1-a2)/nf);
  if (nf < 1E-14)
    return 1;
  return 0;
}

static double
floor_with_tolerance(double v)
{
  double fl;
  fl = floor(v);
  if (((fl + 1.0) / v) < 1.00000000000001)
  {
    return (fl + 1.0);
  }
  return fl;
}
