/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz> (original developer)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "mozCellML.h"
#include "nsWeakReference.h"
#include "mozIPointToZincLocal.h"
#include "nsIRunnable.h"

/* NB: Zinc was the cmgui bridge up until cmgui calls were made across CORBA. */

class mozCellMLPointToZinc
  : public mozIPointToZincLocal,
    public nsSupportsWeakReference,
    public nsIRunnable
{
public:
  mozCellMLPointToZinc
  (
   mozICellML* aCellML,
   mozIScaleWatcher* scaleWatcher,
   double startx, double stopx, double gridx,
   PRUint32 numvariables,
   mozICellMLVariable** variables,
   CmguiIFunction** functions,
   CmguiICommandData* cmddata,
   CmguiITimeSequencePackage* seqPackage,
   CmguiINodeFieldCreator* nfc,
   CmguiINode* pointNode
  );
  NS_DECL_ISUPPORTS
  NS_DECL_MOZIPOINTWATCHER
  NS_DECL_MOZIPOINTTOZINC
  NS_DECL_MOZIPOINTTOZINCLOCAL
  NS_DECL_NSIRUNNABLE
private:
  ~mozCellMLPointToZinc();

  nsresult SendPointsToZinc();

  nsCOMPtr<mozICellML> mCellML;
  double startx, stopx, gridx;
  PRUint32 numvariables;
  PRUint32 unsentPoints;
  nsCOMArray<mozICellMLVariable> variables;
  nsCOMArray<CmguiIFunction> functions;
  nsCOMPtr<mozIScaleWatcher> scaleWatcher;
  nsCOMPtr<CmguiICommandData> cmddata;
  nsCOMPtr<CmguiITimeSequencePackage> seqPackage;
  nsCOMPtr<CmguiINodeFieldCreator> nfc;
  nsCOMPtr<CmguiINode> pointNode;
  PRUint32 point, maxpoint;
  PRUint32* mapIntgToUser, * mapUserToIntg;
  double* data, * scales;
  PRUint32 progressGap;
  mozIGeneralPlugin* plug;

  static nsCOMArray<mozIGeneralPlugin> plugins;
};
