function InitGraphEditor(box)
{
  this.activeModel = null;
  this.activeGraph = null;
  this.notUser = 0;
  var hbl = box.getElementsByTagName("hbox");
  var hb1 = hbl.item(0);
  this.graphlist = hb1.getElementsByTagName("menulist").item(0);
  this.glpop = this.graphlist.getElementsByTagName("menupopup").item(0);
  var bl = hb1.getElementsByTagName("button");
  this.newGraphButton = bl.item(0);
  this.deleteGraphButton = bl.item(1);
  this.deleteGraphButton.setAttribute("disabled", "true");
  var hb2 = hbl.item(1);
  bl = hb2.getElementsByTagName("button");
  this.deleteTraceButton = bl.item(0);
  this.tracelist = hb2.getElementsByTagName("menulist").item(0);
  this.tlpop = this.tracelist.getElementsByTagName("menupopup").item(0);
  var hb3 = hbl.item(2);
  var tl = hb3.getElementsByTagName("tree");
  this.xtree = tl.item(0);
  this.ytree = tl.item(1);
  this.deleteTraceButton.setAttribute("disabled", "true");
  box.vwin = this;
  this.rdfserv = window.rdfserv;
  this.rdfcu = window.rdfcu;

  this.xtree = this.xtree
                   .QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  this.ytree = this.ytree.QueryInterface(Components.interfaces.nsIDOMXULTreeElement);
  this.rdfindex = this.rdfserv.GetResource
    ("http://www.physiome.org/rdf/mozCellML#variableIndex");

  var hb4 = hbl.item(3);
  this.treeCheck = hb4.getElementsByTagName("checkbox").item(0);

  this.colourPicker = hb4.getElementsByTagName("colorpicker").item(0);

  box.vwin = this;

  // Set up the VWin to receive graph notifications...
  this.newGraphTypeCreated = GraphEditorNewGraphType;
  this.graphTypeDeleted = GraphEditorGraphTypeDeleted;
  this.traceDeleted = GraphEditorTraceDeleted;
  this.traceCreated = GraphEditorTraceCreated;
  this.traceVariablesChanged = GraphEditorTraceVariablesChanged;
  this.traceColourChanged = GraphEditorTraceColourChanged;
}

function GraphEditorUpdateModel(model)
{
  // Graphs are per XMLModelInstance, *not* per normal ModelInstance...
  var mod = model.xmlmi;
  var doc = this.glpop.ownerDocument;

  this.editgraph_Finish(true);

  if (mod == this.activeModel)
    return;
  this.activeModel = mod;

  if (this.activeModel.graphs == null)
    this.activeModel.graphs = [];

  if (this.activeModel.last_graph_id == null)
    this.activeModel.last_graph_id = 0;

  // Wipe out all graphs...
  var l = this.glpop.getElementsByTagName("menuitem");
  var i;
  while (l.length > 0)
    this.glpop.removeChild(l.item(0));
  
  // Add all graphs...
  var gi;
  var anySelected = false;
  var selId = -1;
  if (this.activeModel.selectedGraph != null)
    anySelected = true;

  this.notUser++;
  if (this.activeModel.draw_as_tree)
    this.treeCheck.setAttribute("checked", "true");
  else
    this.treeCheck.removeAttribute("checked");
  this.notUser--;

  for (gi in this.activeModel.graphs)
  {
    var g = this.activeModel.graphs[gi];
    var cn = doc.createElement("menuitem");
    cn.setAttribute("label", g.name);
    cn.setAttribute("value", g.id);
    // Select the first graph, if >1...
    if (!anySelected || (this.activeModel.selectedGraph != null &&
                         this.activeModel.selectedGraph == g.id))
    {
      cn.setAttribute("selected", "true");
      GraphSelected(this, gi);
      anySelected = true;
      selId = gi;
    }
    this.glpop.appendChild(cn);
  }
  if (selId >= 0)
  {
    var gl = this.graphlist;
    this.graphlist.QueryInterface(Components.interfaces
                                            .nsIDOMXULSelectControlElement
       ).value = selId;
    this.graphlist.setAttribute("value", selId);
  }
  else
  {
    this.graphlist.setAttribute("disabled", "true");
    this.tracelist.setAttribute("disabled", "true");
  }

  if (this.activeModel.graph_list_monitors == null)
    this.activeModel.graph_list_monitors = [];

  this.activeModel.graph_list_monitors.push(this);
}

function GraphEditorFinish(destroy)
{
  // If we have an active model, remove ourselves from the graph list monitor list.
  var x;
  if (this.activeModel != null)
  {
    for (x in this.activeModel.graph_list_monitors)
    {
      if (this.activeModel.graph_list_monitors[x] == this)
        delete this.activeModel.graph_list_monitors[x];
    }
  }
  // Also, if we have an active graph, remove ourself from the graph monitor list...
  if (this.activeGraph != null)
  {
    GraphFinished(this, this.activeGraph);
  }
}

function NewGraphType(vwin)
{
  if (vwin.activeModel == null)
  {
    GetStatusBar().setAttribute("label", "Please select a model first.");
    return;
  }
  var name = prompt("What should the new graph be called?");
  if (name == null || name == false)
    return;

  vwin.notUser++;
  var graphid = vwin.activeModel.last_graph_id + 1;
  vwin.activeModel.graphs[graphid] = new Graph(name, graphid);

  // Having changed the data structures, inform all UIs depending on it...
  var x;
  for (x in vwin.activeModel.graph_list_monitors)
  {
    vwin.activeModel.graph_list_monitors[x].newGraphTypeCreated(name, graphid);
  }

  // Now make the new graph active...

  // The new graph then becomes active...
  var gle = vwin.graphlist
                .QueryInterface(Components.interfaces
                                          .nsIDOMXULSelectControlElement);
  gle.value = graphid;
  vwin.graphlist.setAttribute("value", graphid);

  GraphSelected(vwin, graphid);
  vwin.notUser--;
}

function GraphEditorNewGraphType(name, graphid)
{
  var doc = this.glpop.ownerDocument;
  var cn = doc.createElement("menuitem");
  this.activeModel.last_graph_id = graphid;
  cn.setAttribute("label", name);
  cn.setAttribute("value", graphid);
  this.glpop.appendChild(cn);
  if (this.activeGraph == null)
  {
    GraphSelected(this, graphid);
    this.notUser++;
    this.graphlist.QueryInterface(Components.interfaces
                                            .nsIDOMXULSelectControlElement
       ).value = graphid;
    this.notUser--;
  }
}

function DeleteGraphType(vwin)
{
  if (vwin.activeModel == null || vwin.activeGraph == null)
  {
    GetStatusBar().setAttribute("label", "No graph selected.");
    return;
  }
  // We now have a graph ready to delete.
  var g = vwin.activeGraph;
  var delid = vwin.activeGraph.id;
  var cg;
  for (cg in vwin.activeModel.graphs)
  {
    if (vwin.activeModel.graphs[cg] == g)
      delete vwin.activeModel.graphs[cg];
  }

  // Having changed the data structures, inform all UIs depending on it...
  var x;
  for (x in vwin.activeModel.graph_list_monitors)
    vwin.activeModel.graph_list_monitors[x].graphTypeDeleted(g);
}

function GraphEditorGraphTypeDeleted(graph)
{
  // Firstly, remove the graph from the drop-down list...
  var l = this.glpop.getElementsByTagName("menuitem");
  var i;
  for (i = 0; i < l.length; i++)
  {
    var ii = l.item(i);
    if (ii.getAttribute("value") != graph.id)
      continue;
    this.glpop.removeChild(ii);
    break;
  }

  // If the deleted graph was inactive, we are done...
  if (this.activeGraph != graph)
    return;

  // See if we can find another graph to make active...
  var replace = null;
  for (i in this.activeModel.graphs)
  {
    replace = this.activeModel.graphs[i];
    break;
  }

  // If there is no replacement graph, disable everything...
  if (replace == null)
  {
    this.deleteGraphButton.setAttribute("disabled", "true");
    this.deleteTraceButton.setAttribute("disabled", "true");
    this.graphlist.setAttribute("disabled", "true");
    this.tracelist.setAttribute("disabled", "true");
    return;
  }

  // Simulate a selection of the replacement graph...
  this.graphlist.QueryInterface(Components.interfaces
                                .nsIDOMXULSelectControlElement
                               ).value = replace.id;
  GraphSelected(this, replace.id);
}

function GraphFinished(w, g)
{
  for (x in g.graph_monitors)
  {
    if (g.graph_monitors[x] == w)
      delete g.graph_monitors[x];
  }

  // Serialise the X and Y tree state.
  g.xtreeState = StringifyTreeState(w.xtree);
  g.ytreeState = StringifyTreeState(w.ytree);
}

function GraphSelected(w, id)
{
  if (w.activeModel == null)
    return;

  if (w.activeGraph != null)
    GraphFinished(w, w.activeGraph);

  w.activeGraph = w.activeModel.graphs[id];
  if (w.activeGraph == null)
    return;

  w.activeModel.selectedGraph = id;
  if (w.activeGraph.graph_monitors == null)
    w.activeGraph.graph_monitors = [];
  w.activeGraph.graph_monitors.push(w);

  w.tracelist.QueryInterface(Components.interfaces
                             .nsIDOMXULSelectControlElement
                                ).value = -1;
  w.tracelist.setAttribute("value", "-1");

  w.deleteGraphButton.removeAttribute("disabled");
  w.graphlist.removeAttribute("disabled");
  w.tracelist.removeAttribute("disabled");

  // Wipe out the traces from the current graph...
  var l = w.tlpop.getElementsByTagName("menuitem");
  var i;
  // Start at 1 not zero, to avoid wiping out the special "New" entry.
  while (l.length > 1)
    w.tlpop.removeChild(l.item(1));

  var doc = w.glpop.ownerDocument;

  var tracecount = 0;
  // Now we need to add new traces to this graph...
  for (tn in w.activeGraph.traces)
  {
    tracecount++;
    var t = w.activeGraph.traces[tn];
    var cn = doc.createElement("menuitem");
    cn.setAttribute("label", t.yvarlev.name + " vs " + t.xvarlev.name);
    cn.setAttribute("value", parseInt(tn) + 1);
    w.tlpop.appendChild(cn);
  }

  if (w.activeGraph.selectedTrace != null)
  {
    w.tracelist.QueryInterface(Components.interfaces
                                         .nsIDOMXULSelectControlElement
       ).value = w.activeGraph.selectedTrace;
  }

  // Having done that, we need to populate the variable box...
  var rsxCT = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                      .createInstance(Components.interfaces.nsIRDFDataSource);
  var rsyCT = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                      .createInstance(Components.interfaces.nsIRDFDataSource);
  var rsxCF = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                      .createInstance(Components.interfaces.nsIRDFDataSource);
  var rsyCF = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                      .createInstance(Components.interfaces.nsIRDFDataSource);
  // XXX can we get rid of these?
  var rsxAT = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                      .createInstance(Components.interfaces.nsIRDFDataSource);
  var rsyAT = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                      .createInstance(Components.interfaces.nsIRDFDataSource);
  var rsxAF = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                      .createInstance(Components.interfaces.nsIRDFDataSource);
  var rsyAF = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                      .createInstance(Components.interfaces.nsIRDFDataSource);

  w.modlevX = new ModelLevel(w.activeModel, w.rdfserv, rsxAT, rsxCT, rsxAF, rsxCF, w.rdfcu);
  w.modlevY = new ModelLevel(w.activeModel, w.rdfserv, rsyAT, rsyCT, rsyAF, rsyCF, w.rdfcu);
  w.treeSourceX = rsxCT;
  w.treeSourceY = rsyCT;
  w.flatSourceX = rsxCF;
  w.flatSourceY = rsyCF;

  var allvars = new Object();
  var numvars = new Object();
  w.activeModel.cellml.getAllVariables(Components.interfaces.mozICellML.NO_IN_VARIABLES,
                                       allvars, numvars);
  numvars = numvars.value;
  w.vlevs = [];
  var cellmlvar, i;
  for (i = 0; i < numvars; i++)
  {
    cellmlvar = allvars.value[i];
    w.vlevs[i] = new VariableLevel(w.modlevX, cellmlvar.name, cellmlvar, i);
    new VariableLevel(w.modlevY, cellmlvar.name, cellmlvar, i);
  }

  /* We only want to see variables which make the units consistent. Note that
   * because we always start with the create new trace option, cutoff=0, or
   * otherwise the new entry could mismatch a single existing one.
   */
  var cnt = 0;
  if (tracecount == 0 ||
      (tracecount == 1 && w.activeGraph.selectedTrace != null &&
       w.activeGraph.selectedTrace > 0))
    cnt = 1;

  w.activeGraph.updateAxisCompatibility(w.modlevX, COMPATIBILITY_XAXIS, cnt);
  w.activeGraph.updateAxisCompatibility(w.modlevY, COMPATIBILITY_YAXIS, cnt);

  // Now add these to the trees...
  if (w.activeXSource != null)
    w.xtree.database.RemoveDataSource(w.activeXSource, true);
  if (w.activeYSource != null)
    w.ytree.database.RemoveDataSource(w.activeYSource, true);

  if (w.treeCheck.hasAttribute("checked"))
  {
    w.activeXSource = rsxCT;
    w.activeYSource = rsyCT;
  }
  else
  {
    w.activeXSource = rsxCF;
    w.activeYSource = rsyCF;
  }

  w.xtree.database.AddDataSource(w.activeXSource);
  w.ytree.database.AddDataSource(w.activeYSource);
  w.xtree.builder.rebuild();
  w.ytree.builder.rebuild();

  if (tracecount != 0 && w.activeGraph.selectedTrace > 0)
  {
    // We have valid X and Y trees, see if we can restore...
    if (w.activeGraph.xtreeState)
      LoadTreeState(w.xtree, w.activeGraph.xtreeState);
    if (w.activeGraph.ytreeState)
      LoadTreeState(w.ytree, w.activeGraph.ytreeState);
  }
}

function ChangeXYTreeDraw(w, cb)
{
  if (w.notUser)
    return;

  if (w.activeXSource != null)
    w.xtree.database.RemoveDataSource(w.activeXSource, true);
  if (w.activeYSource != null)
    w.ytree.database.RemoveDataSource(w.activeYSource, true);
  var ck = (cb.hasAttribute("checked"));

  if (w.activeModel != null)
    w.activeModel.draw_as_tree = ck;
  if (ck)
  {
    w.activeXSource = w.treeSourceX;
    w.activeYSource = w.treeSourceY;
  }
  else
  {
    w.activeXSource = w.flatSourceX;
    w.activeYSource = w.flatSourceY;
  }

  if (w.activeXSource != null)
    w.xtree.database.AddDataSource(w.activeXSource);
  if (w.activeYSource != null)
    w.ytree.database.AddDataSource(w.activeYSource);
  w.xtree.builder.rebuild();
  w.ytree.builder.rebuild();
}

function DeleteTrace(vwin)
{
  var whichTrace = parseInt(vwin.tracelist.getAttribute("value"));

  if (whichTrace == -1)
    return;

  var j;
  for (j = whichTrace + 1; j <= vwin.activeGraph.traces.length; j++)
  {
    // Set the value down by one...
    vwin.activeGraph.traces[j-2] = vwin.activeGraph.traces[j-1];
  }
  vwin.activeGraph.traces.pop();

  vwin.activeGraph.last_trace_id--;

  // The trace is gone. Now inform all graph monitors...
  var x;
  for (x in vwin.activeGraph.graph_monitors)
    vwin.activeGraph.graph_monitors[x].traceDeleted(vwin.activeGraph, whichTrace);
}

function GraphEditorTraceDeleted(graph, whichTrace)
{
  var mi = this.tlpop.getElementsByTagName("menuitem");
  var j;

  var ic = (parseInt(this.tracelist
                         .QueryInterface(Components.interfaces
                                         .nsIDOMXULSelectControlElement)
                 .value) == whichTrace);

  for (j = whichTrace + 1; j <= graph.traces.length + 1; j++)
  {
    // Set the value down by one...
    mi.item(j).setAttribute("value", j - 1);
  }

  var dele = mi.item(whichTrace);
  this.tlpop.removeChild(dele);

  graph.updateAxisCompatibility(this.modlevX, COMPATIBILITY_XAXIS, 0);
  graph.updateAxisCompatibility(this.modlevY, COMPATIBILITY_YAXIS, 0);

  // If the deleted trace is the current trace, go to "New"...
  if (ic)
  {
    this.tracelist.QueryInterface(Components.interfaces
                                  .nsIDOMXULSelectControlElement).value = "-1";
    this.tracelist.setAttribute("value", "-1");
    this.deleteTraceButton.setAttribute("disabled", "true");
  }
}

function GraphVariableChosen(vwin, self)
{
  // Re-set the handlers or we can end up not getting called again(Mozilla bug)
  self.setAttribute("onselect", "GraphVariableChosen(this.parentNode." +
                    "parentNode.parentNode.vwin, this);");

  if (vwin.notUser)
    return;

  // See if we can map the variable into a VariableLevel...
  vwin.rdfsource_active = vwin.activeXSource;
  var xidx = GetRDFIndexFromTree(vwin, vwin.xtree);
  vwin.rdfsource_active = vwin.activeYSource;
  var yidx = GetRDFIndexFromTree(vwin, vwin.ytree);

  // Don't update anything until we have two valid indices...
  if (xidx == -1 || yidx == -1)
    return;

  var xvar = vwin.vlevs[xidx];
  var yvar = vwin.vlevs[yidx];

  var whichTrace = parseInt(vwin.tracelist.getAttribute("value"));

  if (whichTrace == -1)
  {
    // If the trace option is on "New", we need to make a new trace and set it
    // as active...
    var t = new GraphTrace(new AxisVariable(xvar.cellmlvar),
                           new AxisVariable(yvar.cellmlvar));
    vwin.activeGraph.traces.push(t);
    var lti = vwin.activeGraph.last_trace_id + 1;
    vwin.activeGraph.last_trace_id = lti;

    vwin.notUser++;
    var x;
    for (x in vwin.activeGraph.graph_monitors)
      vwin.activeGraph.graph_monitors[x]
          .traceCreated(vwin.activeGraph, lti,
                        t.xvarlev.name, t.yvarlev.name);

    // Now select the new trace...
    var tle = vwin.tracelist.QueryInterface
      (Components.interfaces.nsIDOMXULSelectControlElement);
    tle.value = lti;
    vwin.tracelist.setAttribute("value", lti);

    // Turn on the delete trace button...
    vwin.deleteTraceButton.removeAttribute("disabled");

    // Set the colour picker colour...
    t.assignToColourPicker(vwin.colourPicker);

    vwin.activeGraph.selectedTrace = lti;

    vwin.notUser--;
    return;
  }

  // If we got here, we are editing an existing entry. Get the trace...
  var t = vwin.activeGraph.traces[whichTrace - 1];
  if (t == null)
  {
    alert("traceError");
    return;
  }

  t.xvarlev = new AxisVariable(xvar.cellmlvar);
  t.yvarlev = new AxisVariable(yvar.cellmlvar);

  vwin.notUser++;
  var x;
  for (x in vwin.activeGraph.graph_monitors)
    vwin.activeGraph.graph_monitors[x].traceVariablesChanged
    (vwin.activeGraph, whichTrace, t.xvarlev.name, t.yvarlev.name);
  vwin.notUser--;
}

function GraphEditorTraceCreated(graph, traceId, xname, yname)
{
  var doc = this.glpop.ownerDocument;
  var cn = doc.createElement("menuitem");
  cn.setAttribute("label", yname + " vs " + xname);
  cn.setAttribute("value", traceId);
  this.tlpop.appendChild(cn);
}

function GraphEditorTraceVariablesChanged(graph, traceId, xname, yname)
{
  var el = this.tlpop.getElementsByTagName("menuitem").item(traceId);
  el.setAttribute("label", yname + " vs " + xname);

  this.notUser++;
  // Now make sure that the correct variables show in the UI...
  if (traceId == parseInt(this.tracelist.getAttribute("value")))
  {
    // We have to do this just to trigger an update on the UI...
    this.tracelist.value = -1;
    this.tracelist.value = traceId;

    // Fix what they can see...
    graph.updateAxisCompatibility(this.modlevX, COMPATIBILITY_XAXIS, 1);
    graph.updateAxisCompatibility(this.modlevY, COMPATIBILITY_YAXIS, 1);

    // Now show select the right variable...
    var t = this.activeGraph.traces[traceId - 1];
    var xvl = this.modlevX.imports[t.xvarlev.impno]
                  .components[t.xvarlev.compName].variables[t.xvarlev.name];
    var yvl = this.modlevY.imports[t.yvarlev.impno]
                  .components[t.yvarlev.compName].variables[t.yvarlev.name];
    xvl.selectInTree(this.xtree);
    yvl.selectInTree(this.ytree);
  }
  else
  {
    graph.updateAxisCompatibility(this.modlevX, COMPATIBILITY_XAXIS, 0);
    graph.updateAxisCompatibility(this.modlevY, COMPATIBILITY_YAXIS, 0);
  }
  this.notUser--;
}

function TraceSelected(w, whichTrace)
{
  if (whichTrace == -1)
  {
    w.notUser++;
    w.activeGraph.updateAxisCompatibility(w.modlevX, COMPATIBILITY_XAXIS, 0);
    w.activeGraph.updateAxisCompatibility(w.modlevY, COMPATIBILITY_YAXIS, 0);
    w.xtree.builder.rebuild();
    w.ytree.builder.rebuild();
    w.deleteTraceButton.setAttribute("disabled", "true");
    w.activeGraph.selectedTrace = -1;
    w.notUser--;
    return;
  }

  var t = w.activeGraph.traces[whichTrace - 1];
  if (t == null)
    return;

  w.deleteTraceButton.removeAttribute("disabled");
  t.assignToColourPicker(w.colourPicker);

  w.activeGraph.selectedTrace = whichTrace;

  w.notUser++;
  w.activeGraph.updateAxisCompatibility(w.modlevX, COMPATIBILITY_XAXIS, 1);
  w.activeGraph.updateAxisCompatibility(w.modlevY, COMPATIBILITY_YAXIS, 1);
  w.xtree.builder.rebuild();
  w.ytree.builder.rebuild();

  var xvl, yvl;

  // Need to find xvarlev and yvarlev...
  var i;
  xvl = w.modlevX.imports[t.xvarlev.impno]
                 .components[t.xvarlev.compName].variables[t.xvarlev.name];
  yvl = w.modlevY.imports[t.yvarlev.impno]
                 .components[t.yvarlev.compName].variables[t.yvarlev.name];
  xvl.selectInTree(w.xtree);
  yvl.selectInTree(w.ytree);
  w.notUser--;
}

function GraphColourChange(w)
{
  var newcol = w.colourPicker.getAttribute("color");
  var whichTrace = parseInt(w.tracelist.getAttribute("value"));

  if (whichTrace == -1)
  {
    GetStatusBar().setAttribute("label", "Please select the variables first.");
    return;
  }

  var t = w.activeGraph.traces[whichTrace - 1];
  t.parseColour(newcol);

  // Now inform everyone that the colour changed...
  var x;
  for (x in w.activeGraph.graph_monitors)
    w.activeGraph.graph_monitors[x].traceColourChanged(w.activeGraph, whichTrace, t);
}

function GraphEditorTraceColourChanged(graph, traceId, trace)
{
  trace.assignToColourPicker(this.colourPicker);
}

function GraphEditorSaveContent(node)
{
  var tlv = this.tracelist.QueryInterface(Components.interfaces
                                          .nsIDOMXULSelectControlElement
                                         ).value;
  var glv = this.graphlist.QueryInterface(Components.interfaces
                                          .nsIDOMXULSelectControlElement
                                         ).value;
  node.setAttribute("selectedGraph", glv);
  node.setAttribute("selectedTrace", tlv);
  node.setAttribute("xtreeState", StringifyTreeState(this.xtree));
  node.setAttribute("ytreeState", StringifyTreeState(this.ytree));
}

function GraphEditorRestoreContent(node)
{
  var glv = node.getAttribute("selectedGraph");
  var tlv = node.getAttribute("selectedTrace");
  this.graphlist.QueryInterface(Components.interfaces
                                .nsIDOMXULSelectControlElement
                               ).value = glv;
  GraphSelected(this, glv);
  this.tracelist.QueryInterface(Components.interfaces
                                .nsIDOMXULSelectControlElement
                               ).value = tlv;
  TraceSelected(this, tlv);
  LoadTreeState(this.xtree, node.getAttribute("xtreeState"));
  LoadTreeState(this.ytree, node.getAttribute("ytreeState"));
}
