function getenv(name)
{
  var environment =
  Components.classes["@mozilla.org/process/environment;1"]
            .getService(Components.interfaces.nsIEnvironment);
  return environment.get(name);
}

function FindExecutable(name)
{
  var path = getenv("PATH");

  var sep = ":", psep = "/";
  if (navigator.platform.substring(0, 5) == "Win32")
  {
    sep = ";";
    psep = "\\";
    name = name + ".exe"
  }

  // We implicitly add the following to the search path...
  //  1) The local component directory/gcc/bin
  //  2) The profile directory for this extension/gcc/bin
  Components.classes["@mozilla.org/process/environment;1"]
            .getService(Components.interfaces.nsIEnvironment);
  var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components
                                         .interfaces
                                         .nsIProperties);
  var comsd = dirService.get("ComsD", Components.interfaces.nsIFile);
  comsd.append("gcc");
  comsd.append("bin");
  if (comsd.exists())
    path = (comsd.path) + sep + path;
  var comsd = dirService.get("ProfD", Components.interfaces.nsIFile);
  comsd.append("extensions");
  comsd.append("{a0d0a9af-bb3d-46c1-a470-8800b0b3f3e2}");
  comsd.append("components");
  comsd.append("gcc");
  comsd.append("bin");
  if (comsd.exists())
    path = (comsd.path) + sep + path;

  var idx;
  while ((idx = path.indexOf(sep)) != -1)
  {
    try
    {
      var subpath = path.substring(0, idx);
      path = path.substring(idx + 1);
      var fullpath = subpath + psep + name;
      var lf = Components.classes["@mozilla.org/file/local;1"]
                         .createInstance(Components.interfaces.nsILocalFile);
      lf.initWithPath(fullpath);
      if (lf.exists())
        return fullpath;
    }
    catch (e)
    {
    }
  }
  return null;
}

function GetVariableTreePref()
{
  var prefservice =
  Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefService);
  var cellBranch = prefservice.getBranch("physiome.mozcellml.");
  try
  {
    return cellBranch.getBoolPref("useTreeLayout");
  }
  catch (e)
  {
    cellBranch.setBoolPref("useTreeLayout", true);
    return true;
  }
}

function GetCompiler()
{
  var prefservice =
  Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefService);
  var cellBranch = prefservice.getBranch("physiome.mozcellml.");
  var compline, cc;
  try
  {
    compline = cellBranch.getCharPref("compiler");
    if (compline == "" || compline == null)
      throw "Compiler is empty";
  }
  catch (e)
  {
    cc = FindExecutable("gcc");
    if (cc == null)
    {
      cc = "/usr/bin/gcc";
    }
    else // it might have spaces in it...
      cc = "\"" + cc + "\"";

    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                               .getService(Components
                                           .interfaces
                                           .nsIProperties);
    var components = dirService.get("ComsD", Components.interfaces.nsIFile);
    var compTest = components.clone();
    compTest.append("DoIntegration.I");
    if (!compTest.exists())
    {
      components = dirService.get("ProfD", Components.interfaces.nsIFile);
      compTest = components.clone();
      compTest.append("DoIntegration.I");
      if (!compTest.exists())
      {
        components.append("extensions");
        components.append("{a0d0a9af-bb3d-46c1-a470-8800b0b3f3e2}");
        components.append("components");
      }
    }
    var includePath = "-I\"" + components.path + "\"";

    var compline = cc + " -O3 " + includePath + " -lnspr4";
    if (navigator.platform.substring(0, 5) == "Win32")
      compline = compline + " -Wl,--subsystem -Wl,windows";
    else
      compline = compline + " -lm";
    cellBranch.setCharPref("compiler", compline);
  }
  return compline;
}

function SetVisibility(x, isvis)
{
  x.setAttribute("style", "visibility: " + (isvis ? "visible" : "collapse"));
}

function QueryVariable(sup)
{
  return sup.QueryInterface(Components.interfaces.mozCellMLIVariable);
}

function ModelLoadObserver()
{
  return this;
}

ModelLoadObserver.prototype =
{
  QueryInterface: function (aIID)
  {
    if (
        aIID.equals(Components.interfaces.nsIStreamLoaderObserver) ||
        aIID.equals(Components.interfaces.nsISupports)
       )
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  onStreamComplete: function (loader, ctxt, status, resultLength, result)
  {
    var strb = document.getElementById("mozcellml-string-bundle");
    if (status == 0)
      this.statusBar.label = strb.getString("documentLoaded");
    else
      this.statusBar.label = strb.getString("loadModelURLFailed");
  }
};

function UserRequestLoad(location)
{
  var strb = window.top.document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById('status-bar-display');
  if (statusBar == null)
    statusBar = GetStatusBar();
  statusBar.setAttribute("label", strb.getFormattedString("loadingURLLocation", [(""+ location)]));

  location = "mozcellml:" + location;

  var url = Components.classes["@mozilla.org/network/standard-url;1"]
                      .createInstance(Components.interfaces.nsIURI);
  
  var chan = Components.classes["@mozilla.org/network/io-service;1"]
                       .getService(Components.interfaces.nsIIOService)
                       .newChannel(location, null, null);
  url.spec = location;
  chan.originalURI = url;

  window.loader = Components.classes["@mozilla.org/network/stream-loader;1"]
                            .createInstance(Components.interfaces.nsIStreamLoader);
  var mlo = new ModelLoadObserver();
  mlo.statusBar = statusBar;
  window.loader.init(chan, mlo, null);
}

function UserRequestValidate(location)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  var statusBar = document.getElementById('status-bar-display');
  statusBar.label = strb.getFormattedString("loadingURLLocation", [("" + location)]);
  location = "mozcellml-validate:" + location;

  var url = Components.classes["@mozilla.org/network/standard-url;1"]
                      .createInstance(Components.interfaces.nsIURI);
  var chan = Components.classes["@mozilla.org/network/io-service;1"]
                       .getService(Components.interfaces.nsIIOService)
                       .newChannel(location, null, null);
  url.spec = location;
  chan.originalURI = url;

  window.loader = Components.classes["@mozilla.org/network/stream-loader;1"]
                            .createInstance(Components.interfaces.nsIStreamLoader);
  var mlo = new ModelLoadObserver();
  mlo.statusBar = statusBar;
  window.loader.init(chan, mlo, null);
}

function SelectLocalModel(returnto)
{
  var strb = document.getElementById("mozcellml-string-bundle");
  try
  {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                       .createInstance(nsIFilePicker);
    fp.init(window, strb.getString("chooseLoadModel"),
            nsIFilePicker.modeOpen);
    fp.appendFilter(strb.getString("allCellML"), "*.cml; *.xml");
    fp.appendFilter("CML", "*.cml");
    fp.appendFilter("XML", "*.xml");
    fp.appendFilter(strb.getString("allFiles"), "*");
    if (fp.show() != nsIFilePicker.returnOK)
      return;
    returnto.value = fp.fileURL.spec;
  }
  catch (ex)
  {
  }
}
