/*	-*- C++ -*-
 * Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */

#include "corba_poa.h"

#include "corba_locator.h"
#include "corba_proxy.h"

#include <nsISupports.h>


CORBA_POA *the_corba_poa = NULL;

CORBA_POA::CORBA_POA()
{
  the_corba_poa = this;
}

CORBA_POA* get_corba_poa()
{
  return the_corba_poa;
}

CORBA_POA::~CORBA_POA()
{
}


void CORBA_POA::init(int argc, char *argv[])
{
  orb_ = CORBA::ORB_init(argc, argv);

  CORBA::Object_var obj = orb_->resolve_initial_references("RootPOA");
  PortableServer::POA_var root_poa = PortableServer::POA::_narrow(obj);
  PortableServer::POAManager_var pman = root_poa->the_POAManager();

  CORBA::PolicyList policies;
  policies.length(5);
  policies[0] =
    root_poa->create_servant_retention_policy(PortableServer::NON_RETAIN);
  policies[1] =
    root_poa->create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
  policies[2] =
    root_poa->create_id_assignment_policy(PortableServer::USER_ID);
  // the next one should be the default
  policies[3] =
    root_poa->create_implicit_activation_policy(PortableServer::NO_IMPLICIT_ACTIVATION);
  policies[4] =
    root_poa->create_thread_policy(PortableServer::ORB_CTRL_MODEL);

  xpcom_poa_ = root_poa->create_POA("XPCOM", pman, policies);

  proxy_locator_ = new CORBA_Proxy_Locator();

  xpcom_poa_->set_servant_manager(proxy_locator_->_this());

  pman->activate();
}
