/*	-*- C++ -*-
 * Copyright (C) 2001-2002,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */
#ifndef HEADER__XPCOM2CORBA__INCLUDED
#define HEADER__XPCOM2CORBA__INCLUDED

#include "corba_poa.h"

#include <CORBA.h>
#include <nsISupports.h>
#include <nsString.h>
#include <nsCRT.h>

#include <ICORBAWrapper.h>


inline void xpcom2corba(PRBool xpcom_val, CORBA::Boolean &corba_val)
{
  corba_val = (xpcom_val != PR_FALSE);
}

inline void xpcom2corba(PRUint8 xpcom_val, CORBA::Octet &corba_val)
{
  corba_val = xpcom_val;
}

inline void xpcom2corba(PRInt8 xpcom_val, CORBA::Char &corba_val)
{
  corba_val = xpcom_val;
}

inline void xpcom2corba(PRUint16 xpcom_val, CORBA::UShort &corba_val)
{
  corba_val = xpcom_val;
}

inline void xpcom2corba(PRInt16 xpcom_val, CORBA::Short &corba_val)
{
  corba_val = xpcom_val;
}

inline void xpcom2corba(PRUint32 xpcom_val, CORBA::ULong &corba_val)
{
  corba_val = xpcom_val;
}

inline void xpcom2corba(PRInt32 xpcom_val, CORBA::Long &corba_val)
{
  corba_val = xpcom_val;
}

inline void xpcom2corba(double xpcom_val, CORBA::Double &corba_val)
{
  corba_val = xpcom_val;
}

inline void xpcom2corba(float xpcom_val, CORBA::Float &corba_val)
{
  corba_val = xpcom_val;
}

inline void xpcom2corba(const char *xpcom_val, char *&corba_val)
{
  corba_val = CORBA::string_dup(xpcom_val);
}

inline void xpcom2corba(const PRUnichar *xpcom_val, CORBA::WChar *&corba_val)
{
  // TODO: actually we would have to convert from UCS-2 to UTF-16...
  const size_t len = nsCRT::strlen(xpcom_val);

  corba_val = CORBA::wstring_alloc(len);

  for (size_t i = 0; i < len; ++i)
  {
    corba_val[i] = xpcom_val[i];
  }
}

inline void xpcom2corba(const PRUnichar *xpcom_val,
			CORBA::WString_out corba_val)
{
  ::xpcom2corba(xpcom_val, corba_val);
}

inline void xpcom2corba(const nsACString &xpcom_val, char *&corba_val)
{
  corba_val = CORBA::string_dup(PromiseFlatCString(xpcom_val).get());
}

inline void xpcom2corba(const nsACString &xpcom_val,
			CORBA::String_out corba_val)
{
  corba_val = CORBA::string_dup(PromiseFlatCString(xpcom_val).get());
}

inline void xpcom2corba(const nsACString &xpcom_val,
			CORBA::WChar *&corba_val)
{
  // TODO
}

inline void xpcom2corba(const nsACString &xpcom_val,
			CORBA::WString_out corba_val)
{
  ::xpcom2corba(xpcom_val, corba_val);
}

extern CORBA_POA* get_corba_poa();

template<class TypeXPCOM, class TypeCORBA>
void xpcom2corba(TypeXPCOM *xpcom_obj, TypeCORBA*, typename TypeCORBA::_ptr_type* t)
{
  if (xpcom_obj)
  {
    nsCOMPtr<ICORBAWrapper> corba_wrapper = do_QueryInterface(xpcom_obj);

    if (!corba_wrapper)
    {
      TypeCORBA* dummy = (TypeCORBA*)NULL;
      *t = get_corba_poa()->CORBA_POA::register_object(xpcom_obj, dummy);
    }
    else
    {
      CORBA::Object_ptr corba_obj;

      corba_wrapper->GetCorbaObj(&corba_obj);
      typename TypeCORBA::_ptr_type narrowed_obj = TypeCORBA::_narrow(corba_obj);
      CORBA::release(corba_obj);

      *t = narrowed_obj;
    }
  }
  else
  {
    *t = TypeCORBA::_duplicate(TypeCORBA::_nil());
  }
}


#endif
